<?php

namespace Cleancrm\Advancedcrm\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Cleancrm\Advancedcrm\Http\Requests\LeadRequest;
use Cleancrm\Advancedcrm\Interfaces\LeadRepositoryInterface;
use Cleancrm\Advancedcrm\Models\Lead;

/**
 * Resource controller class for lead.
 */
class LeadResourceController extends BaseController
{

    /**
     * Initialize lead resource controller.
     *
     * @param type LeadRepositoryInterface $lead
     *
     * @return null
     */
    public function __construct(LeadRepositoryInterface $lead)
    {
        parent::__construct();
        $this->repository = $lead;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Cleancrm\Advancedcrm\Repositories\Criteria\LeadResourceCriteria::class);
    }

    /**
     * Display a list of lead.
     *
     * @return Response
     */
    public function index(LeadRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Cleancrm\Advancedcrm\Repositories\Presenter\LeadPresenter::class)
                ->$function();
        }

        $leads = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('advancedcrm::lead.names'))
            ->view('advancedcrm::lead.index', true)
            ->data(compact('leads', 'view'))
            ->output();
    }

    /**
     * Display lead.
     *
     * @param Request $request
     * @param Model   $lead
     *
     * @return Response
     */
    public function show(LeadRequest $request, Lead $lead)
    {

        if ($lead->exists) {
            $view = 'advancedcrm::lead.show';
        } else {
            $view = 'advancedcrm::lead.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('advancedcrm::lead.name'))
            ->data(compact('lead'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new lead.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(LeadRequest $request)
    {

        $lead = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('advancedcrm::lead.name')) 
            ->view('advancedcrm::lead.create', true) 
            ->data(compact('lead'))
            ->output();
    }

    /**
     * Create new lead.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(LeadRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $lead                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('advancedcrm::lead.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('advancedcrm/lead/' . $lead->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/advancedcrm/lead'))
                ->redirect();
        }

    }

    /**
     * Show lead for editing.
     *
     * @param Request $request
     * @param Model   $lead
     *
     * @return Response
     */
    public function edit(LeadRequest $request, Lead $lead)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('advancedcrm::lead.name'))
            ->view('advancedcrm::lead.edit', true)
            ->data(compact('lead'))
            ->output();
    }

    /**
     * Update the lead.
     *
     * @param Request $request
     * @param Model   $lead
     *
     * @return Response
     */
    public function update(LeadRequest $request, Lead $lead)
    {
        try {
            $attributes = $request->all();

            $lead->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('advancedcrm::lead.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('advancedcrm/lead/' . $lead->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('advancedcrm/lead/' . $lead->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the lead.
     *
     * @param Model   $lead
     *
     * @return Response
     */
    public function destroy(LeadRequest $request, Lead $lead)
    {
        try {

            $lead->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('advancedcrm::lead.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('advancedcrm/lead/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('advancedcrm/lead/' . $lead->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple lead.
     *
     * @param Model   $lead
     *
     * @return Response
     */
    public function delete(LeadRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('advancedcrm::lead.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('advancedcrm/lead'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/advancedcrm/lead'))
                ->redirect();
        }

    }

    /**
     * Restore deleted leads.
     *
     * @param Model   $lead
     *
     * @return Response
     */
    public function restore(LeadRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('advancedcrm::lead.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/advancedcrm/lead'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/advancedcrm/lead/'))
                ->redirect();
        }

    }

}
