<?php

namespace Cleancrm\Advancedcrm\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Cleancrm\Advancedcrm\Http\Requests\LeadSourceRequest;
use Cleancrm\Advancedcrm\Interfaces\LeadSourceRepositoryInterface;
use Cleancrm\Advancedcrm\Models\LeadSource;

/**
 * Resource controller class for lead_source.
 */
class LeadSourceResourceController extends BaseController
{

    /**
     * Initialize lead_source resource controller.
     *
     * @param type LeadSourceRepositoryInterface $lead_source
     *
     * @return null
     */
    public function __construct(LeadSourceRepositoryInterface $lead_source)
    {
        parent::__construct();
        $this->repository = $lead_source;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Cleancrm\Advancedcrm\Repositories\Criteria\LeadSourceResourceCriteria::class);
    }

    /**
     * Display a list of lead_source.
     *
     * @return Response
     */
    public function index(LeadSourceRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Cleancrm\Advancedcrm\Repositories\Presenter\LeadSourcePresenter::class)
                ->$function();
        }

        $lead_sources = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('advancedcrm::lead_source.names'))
            ->view('advancedcrm::lead_source.index', true)
            ->data(compact('lead_sources', 'view'))
            ->output();
    }

    /**
     * Display lead_source.
     *
     * @param Request $request
     * @param Model   $lead_source
     *
     * @return Response
     */
    public function show(LeadSourceRequest $request, LeadSource $lead_source)
    {

        if ($lead_source->exists) {
            $view = 'advancedcrm::lead_source.show';
        } else {
            $view = 'advancedcrm::lead_source.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('advancedcrm::lead_source.name'))
            ->data(compact('lead_source'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new lead_source.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(LeadSourceRequest $request)
    {

        $lead_source = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('advancedcrm::lead_source.name')) 
            ->view('advancedcrm::lead_source.create', true) 
            ->data(compact('lead_source'))
            ->output();
    }

    /**
     * Create new lead_source.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(LeadSourceRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $lead_source                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('advancedcrm::lead_source.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('advancedcrm/lead_source/' . $lead_source->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/advancedcrm/lead_source'))
                ->redirect();
        }

    }

    /**
     * Show lead_source for editing.
     *
     * @param Request $request
     * @param Model   $lead_source
     *
     * @return Response
     */
    public function edit(LeadSourceRequest $request, LeadSource $lead_source)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('advancedcrm::lead_source.name'))
            ->view('advancedcrm::lead_source.edit', true)
            ->data(compact('lead_source'))
            ->output();
    }

    /**
     * Update the lead_source.
     *
     * @param Request $request
     * @param Model   $lead_source
     *
     * @return Response
     */
    public function update(LeadSourceRequest $request, LeadSource $lead_source)
    {
        try {
            $attributes = $request->all();

            $lead_source->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('advancedcrm::lead_source.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('advancedcrm/lead_source/' . $lead_source->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('advancedcrm/lead_source/' . $lead_source->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the lead_source.
     *
     * @param Model   $lead_source
     *
     * @return Response
     */
    public function destroy(LeadSourceRequest $request, LeadSource $lead_source)
    {
        try {

            $lead_source->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('advancedcrm::lead_source.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('advancedcrm/lead_source/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('advancedcrm/lead_source/' . $lead_source->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple lead_source.
     *
     * @param Model   $lead_source
     *
     * @return Response
     */
    public function delete(LeadSourceRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('advancedcrm::lead_source.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('advancedcrm/lead_source'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/advancedcrm/lead_source'))
                ->redirect();
        }

    }

    /**
     * Restore deleted lead_sources.
     *
     * @param Model   $lead_source
     *
     * @return Response
     */
    public function restore(LeadSourceRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('advancedcrm::lead_source.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/advancedcrm/lead_source'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/advancedcrm/lead_source/'))
                ->redirect();
        }

    }

}
