<?php

namespace Cleancrm\Advancedcrm\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Cleancrm\Advancedcrm\Http\Requests\OpportunityRequest;
use Cleancrm\Advancedcrm\Interfaces\OpportunityRepositoryInterface;
use Cleancrm\Advancedcrm\Models\Opportunity;

/**
 * Resource controller class for opportunity.
 */
class OpportunityResourceController extends BaseController
{

    /**
     * Initialize opportunity resource controller.
     *
     * @param type OpportunityRepositoryInterface $opportunity
     *
     * @return null
     */
    public function __construct(OpportunityRepositoryInterface $opportunity)
    {
        parent::__construct();
        $this->repository = $opportunity;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Cleancrm\Advancedcrm\Repositories\Criteria\OpportunityResourceCriteria::class);
    }

    /**
     * Display a list of opportunity.
     *
     * @return Response
     */
    public function index(OpportunityRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Cleancrm\Advancedcrm\Repositories\Presenter\OpportunityPresenter::class)
                ->$function();
        }

        $opportunities = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('advancedcrm::opportunity.names'))
            ->view('advancedcrm::opportunity.index', true)
            ->data(compact('opportunities', 'view'))
            ->output();
    }

    /**
     * Display opportunity.
     *
     * @param Request $request
     * @param Model   $opportunity
     *
     * @return Response
     */
    public function show(OpportunityRequest $request, Opportunity $opportunity)
    {

        if ($opportunity->exists) {
            $view = 'advancedcrm::opportunity.show';
        } else {
            $view = 'advancedcrm::opportunity.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('advancedcrm::opportunity.name'))
            ->data(compact('opportunity'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new opportunity.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(OpportunityRequest $request)
    {

        $opportunity = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('advancedcrm::opportunity.name')) 
            ->view('advancedcrm::opportunity.create', true) 
            ->data(compact('opportunity'))
            ->output();
    }

    /**
     * Create new opportunity.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(OpportunityRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $opportunity                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('advancedcrm::opportunity.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('advancedcrm/opportunity/' . $opportunity->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/advancedcrm/opportunity'))
                ->redirect();
        }

    }

    /**
     * Show opportunity for editing.
     *
     * @param Request $request
     * @param Model   $opportunity
     *
     * @return Response
     */
    public function edit(OpportunityRequest $request, Opportunity $opportunity)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('advancedcrm::opportunity.name'))
            ->view('advancedcrm::opportunity.edit', true)
            ->data(compact('opportunity'))
            ->output();
    }

    /**
     * Update the opportunity.
     *
     * @param Request $request
     * @param Model   $opportunity
     *
     * @return Response
     */
    public function update(OpportunityRequest $request, Opportunity $opportunity)
    {
        try {
            $attributes = $request->all();

            $opportunity->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('advancedcrm::opportunity.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('advancedcrm/opportunity/' . $opportunity->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('advancedcrm/opportunity/' . $opportunity->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the opportunity.
     *
     * @param Model   $opportunity
     *
     * @return Response
     */
    public function destroy(OpportunityRequest $request, Opportunity $opportunity)
    {
        try {

            $opportunity->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('advancedcrm::opportunity.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('advancedcrm/opportunity/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('advancedcrm/opportunity/' . $opportunity->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple opportunity.
     *
     * @param Model   $opportunity
     *
     * @return Response
     */
    public function delete(OpportunityRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('advancedcrm::opportunity.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('advancedcrm/opportunity'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/advancedcrm/opportunity'))
                ->redirect();
        }

    }

    /**
     * Restore deleted opportunities.
     *
     * @param Model   $opportunity
     *
     * @return Response
     */
    public function restore(OpportunityRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('advancedcrm::opportunity.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/advancedcrm/opportunity'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/advancedcrm/opportunity/'))
                ->redirect();
        }

    }

}
