<?php

namespace Cleancrm\Advancedcrm\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Cleancrm\Advancedcrm\Http\Requests\RelationRequest;
use Cleancrm\Advancedcrm\Interfaces\RelationRepositoryInterface;
use Cleancrm\Advancedcrm\Models\Relation;

/**
 * Resource controller class for relation.
 */
class RelationResourceController extends BaseController
{

    /**
     * Initialize relation resource controller.
     *
     * @param type RelationRepositoryInterface $relation
     *
     * @return null
     */
    public function __construct(RelationRepositoryInterface $relation)
    {
        parent::__construct();
        $this->repository = $relation;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Cleancrm\Advancedcrm\Repositories\Criteria\RelationResourceCriteria::class);
    }

    /**
     * Display a list of relation.
     *
     * @return Response
     */
    public function index(RelationRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Cleancrm\Advancedcrm\Repositories\Presenter\RelationPresenter::class)
                ->$function();
        }

        $relations = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('advancedcrm::relation.names'))
            ->view('advancedcrm::relation.index', true)
            ->data(compact('relations', 'view'))
            ->output();
    }

    /**
     * Display relation.
     *
     * @param Request $request
     * @param Model   $relation
     *
     * @return Response
     */
    public function show(RelationRequest $request, Relation $relation)
    {

        if ($relation->exists) {
            $view = 'advancedcrm::relation.show';
        } else {
            $view = 'advancedcrm::relation.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('advancedcrm::relation.name'))
            ->data(compact('relation'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new relation.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(RelationRequest $request)
    {

        $relation = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('advancedcrm::relation.name')) 
            ->view('advancedcrm::relation.create', true) 
            ->data(compact('relation'))
            ->output();
    }

    /**
     * Create new relation.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(RelationRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $relation                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('advancedcrm::relation.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('advancedcrm/relation/' . $relation->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/advancedcrm/relation'))
                ->redirect();
        }

    }

    /**
     * Show relation for editing.
     *
     * @param Request $request
     * @param Model   $relation
     *
     * @return Response
     */
    public function edit(RelationRequest $request, Relation $relation)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('advancedcrm::relation.name'))
            ->view('advancedcrm::relation.edit', true)
            ->data(compact('relation'))
            ->output();
    }

    /**
     * Update the relation.
     *
     * @param Request $request
     * @param Model   $relation
     *
     * @return Response
     */
    public function update(RelationRequest $request, Relation $relation)
    {
        try {
            $attributes = $request->all();

            $relation->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('advancedcrm::relation.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('advancedcrm/relation/' . $relation->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('advancedcrm/relation/' . $relation->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the relation.
     *
     * @param Model   $relation
     *
     * @return Response
     */
    public function destroy(RelationRequest $request, Relation $relation)
    {
        try {

            $relation->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('advancedcrm::relation.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('advancedcrm/relation/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('advancedcrm/relation/' . $relation->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple relation.
     *
     * @param Model   $relation
     *
     * @return Response
     */
    public function delete(RelationRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('advancedcrm::relation.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('advancedcrm/relation'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/advancedcrm/relation'))
                ->redirect();
        }

    }

    /**
     * Restore deleted relations.
     *
     * @param Model   $relation
     *
     * @return Response
     */
    public function restore(RelationRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('advancedcrm::relation.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/advancedcrm/relation'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/advancedcrm/relation/'))
                ->redirect();
        }

    }

}
